local videoModes = ...

--    
local DbOption			= require('Options.DbOption')
local i18n				= require('i18n')
local lfs				= require('lfs')

local _ = i18n.ptranslate

local difficultyOptions = {}
local graphicOptions = {}
local viewsOptions = {cockpit = {}}
local viewsCockpitOptions = viewsOptions.cockpit
local soundOptions = {}
local miscOptions = {}

local function difficulty(name)
	difficultyOptions[name] = DbOption.new()
	
	return difficultyOptions[name]
end

local function graphics(name)
	graphicOptions[name] = DbOption.new()
	
	return graphicOptions[name]
end

local function viewsCockpit(name)
	viewsCockpitOptions[name] = DbOption.new()
	
	return viewsCockpitOptions[name]
end

local function sound(name)
	soundOptions[name] = DbOption.new()
	
	return soundOptions[name]
end

local function misc(name)
	miscOptions[name] = DbOption.new()
	
	return miscOptions[name]	
end

-- :
-- combo({	Name('Name 1'):Value(0),
--			Name('Name 2'):Value(1):OnlyArch64(),
--			...			
--})
local Name = DbOption.Item

-- :
-- slider(Range(-30, 0):Mute(-100):OnlyArch64())
--})
local Range = DbOption.Range

difficulty('easyFlight'):setValue(true):checkbox():setEnforceable():low(true):medium(false):high(false)
difficulty('easyRadar'):setValue(true):checkbox():setEnforceable():low(true):medium(false):high(false)
difficulty('radio'):setValue(false):checkbox():setEnforceable():low(true):medium(false):high(false)
difficulty('tips'):setValue(true):checkbox():setEnforceable():low(true):medium(true):high(true)
difficulty('permitCrash'):setValue(true):checkbox():setEnforceable():low(true):medium(true):high(false)
difficulty('easyCommunication'):setValue(true):checkbox():setEnforceable():low(true):medium(true):high(false)
difficulty('padlock'):setValue(true):checkbox():setEnforceable():low(true):medium(true):high(false)
difficulty('labels'):setValue(true):checkbox():setEnforceable():low(true):medium(true):high(false)
difficulty('fuel'):setValue(false):checkbox():setEnforceable():low(false):medium(false):high(false)
difficulty('weapons'):setValue(false):checkbox():setEnforceable():low(false):medium(false):high(false)
difficulty('immortal'):setValue(false):checkbox():setEnforceable():low(false):medium(false):high(false)
difficulty('reports'):setValue(false):checkbox()
difficulty('miniHUD'):setValue(false):checkbox():setEnforceable():low(true):medium(true):high(false)
difficulty('setGlobal'):setValue(false):checkbox()
difficulty('cockpitVisualRM'):setValue(false):checkbox():setEnforceable():low(false):medium(false):high(false)

difficulty('birds'):setValue(0):setEnforceable():slider(Range(0, 1000)):low(0):medium(0):high(0)
difficulty('optionsView'):setValue('optview_all'):setEnforceable():radio({	Name('mapOnly')		:Value('optview_onlymap'),
																			Name('myPlane')		:Value('optview_myaircraft'),
																			Name('allies')		:Value('optview_allies'),
																			Name('alliesOnly')	:Value('optview_onlyallies'),
																			Name('all')			:Value('optview_all'),
																		}):low('optview_all'):medium('optview_all'):high('optview_all')

difficulty('map'):setValue(true):checkbox():low(true):medium(true):high(true)

difficulty('units'):setValue('metric'):combo({	Name(_('IMPERIAL'))	:Value('imperial'),
												Name(_('METRIC'))	:Value('metric'),
											})

difficulty('externalViews'):setValue(true):checkbox():setEnforceable():low(true):medium(true):high(true)
difficulty('userSnapView'):setValue(true):checkbox():low(true):medium(true):high(true)

difficulty('iconsTheme'):setValue('nato'):combo({	Name(_('NATO'))		:Value('nato'),
													Name(_('Russia'))	:Value('russian'),
												})

difficulty('geffect'):setValue('none'):combo({	Name(_('NONE'))			:Value('none'),
												Name(_('REDUCED'))		:Value('reduced'),
												Name(_('REALISTIC'))	:Value('realistic'),
											}):low('none'):medium('reduced'):high('realistic')
                                            
difficulty('impostors'):setValue(0):combo({	Name(_('OFF'))		:Value(''),
											Name(_('SMALL'))	:Value('small'),
											Name(_('MEDIUM'))	:Value('medium'),
                                            Name(_('LARGE'))	:Value('large'),
									}):low(''):medium(''):high('')                                            

graphics('textures'):setValue(0):combo({Name(_('LOW'))		:Value(0),
										Name(_('MEDIUM'))	:Value(1),
										Name(_('HIGH'))		:Value(2)	:OnlyArch64(),
									}):low(0):medium(1):high(2)
                                    
graphics('terrainTextures'):setValue(0):combo({Name(_('LOW'))   :Value('min'),
                                               Name(_('HIGH'))	:Value('max'),
									}):low('min'):medium('min'):high('max')                                    

graphics('flatTerrainShadows'):setValue(0):combo({Name(_('OFF')) :Value(0),
                                                  Name(_('ON'))	:Value(1),
									}):low(1):medium(0):high(0)        

graphics('HDR'):setValue(0):combo({	Name(_('OFF'))			:Value(0),
									Name(_('ON'))	        :Value(1)	:OnlyArch64(),
								})

graphics('civTraffic'):setValue(''):setEnforceable():combo({Name(_('OFF'))		:Value(''),
															Name(_('LOW'))		:Value('low'),
															Name(_('MEDIUM'))	:Value('medium'),
															Name(_('HIGH'))		:Value('high')	:OnlyArch64(),
														}):low(''):medium('medium'):high('high')

graphics('water'):setValue(2):combo({	Name(_('LOW'))		:Value(0),
										Name(_('MEDIUM'))	:Value(1),
										Name(_('HIGH'))		:Value(2),
									}):low(0):medium(1):high(2)

graphics('visibRange'):setValue('Low'):combo({	Name(_('LOW'))		:Value('Low'),
												Name(_('MEDIUM'))	:Value('Medium'),
												Name(_('HIGH'))		:Value('High'),
												Name(_('ULTRA'))	:Value('Ultra')	:OnlyArch64(),
												Name(_('EXTREME'))	:Value('Extreme')	:OnlyArch64(),
											}):low('Low'):medium('Medium'):high('High')
												
graphics('heatBlr'):setValue(0):combo({	Name(_('OFF'))	:Value(0),
										Name(_('LOW'))	:Value(1),
										Name(_('HIGH'))	:Value(2),
									}):low(0):medium(1):high(2)
										
graphics('DOF'):setValue(0):combo({	Name(_('OFF'))	  :Value(0),
									Name(_('BOKEH'))  :Value(1),
									Name(_('SIMPLE')) :Value(2),
									}):low(0):medium(0):high(1)

graphics('shadows'):setValue(2):combo({	
										Name(_('OFF'))		:Value(0),
										Name(_('FLAT ONLY'))	:Value(1),
										Name(_('LOW'))		:Value(2),
										Name(_('MEDIUM'))	:Value(3),
										Name(_('HIGH'))		:Value(4),
										Name(_('ULTRA'))	:Value(5),
									}):low(1):medium(3):high(4)
										
graphics('MSAA'):setValue(2):combo({Name(_('OFF'))	:Value(0),
									Name('2x')		:Value(1),
									Name('4x')		:Value(2),
									Name('8x')		:Value(3)	:OnlyArch64(),
									Name('16x')		:Value(4)	:OnlyArch64(),
									Name('8xQ')		:Value(5)	:OnlyArch64(),
									Name('16xQ')	:Value(6)	:OnlyArch64(),
								})
										
graphics('aspect'):setValue(4/3):combo({Name('4:3')		:Value(4/3),
										Name('3:2')		:Value(3/2),
										Name('5:4')		:Value(5/4),
										Name('16:9')	:Value(16/9),
										Name('16:10')	:Value(16/10),
									})
                                    
graphics('LensEffects'):setValue(3):combo({	Name(_('NONE'))		    :Value(0),
                                            Name(_('DIRT'))	        :Value(1),
                                            Name(_('FLARE_lenseffect','FLARE'))		:Value(2),
                                            Name(_('DIRT+FLARE'))	:Value(3),
                                }):low(2):medium(3):high(3)

graphics('clouds'):setValue(1):combo({	Name(_('LOW'))		   	:Value(0),
										Name(_('MEDIUM'))	    :Value(1),
										Name(_('HIGH'))			:Value(2),
										Name(_('ULTRA'))		:Value(3),
                                }):low(0):medium(1):high(2)

graphics('anisotropy'):setValue(4):combo({Name(_('OFF')):Value(0),
									Name('2x')		:Value(1),
									Name('4x')		:Value(2),
									Name('8x')		:Value(3),
									Name('16x')		:Value(4),
								}):low(4):medium(4):high(4)

local function loadMonitorProfiles(folder, values)
	for fileName in lfs.dir(folder) do
		if '.lua' == string.sub(fileName, -4) then
			local func, err = loadfile(folder .. '/' .. fileName)
			
			if func then
				local env = {screen = {width = 1, height = 1, aspect = 1}}
		
				setfenv(func, env)
				func()
				
				local name = i18n.ptranslate(env.name)
				local value = string.lower(string.sub(fileName, 1, -5))
				
				table.insert(values, Name(name):Value(value))
			else
				print(err)
			end
		end	
	end
end

local function getMultiMonitorSetupValues()
	local values = {}
		
	loadMonitorProfiles(simPath .. 'Config/MonitorSetup/', values)
	loadMonitorProfiles(lfs.writedir() .. 'Config/MonitorSetup/', values)
	
	return values
end
	
graphics('multiMonitorSetup'):setValue('1camera'):combo(getMultiMonitorSetupValues())

local screenWidths = {}
local screenHeights = {}
	
for i, videoMode in ipairs(videoModes) do
	table.insert(screenWidths, videoMode[1])
	table.insert(screenHeights, videoMode[2])
end	

graphics('width'):setValue(1024):combo(screenWidths)
graphics('height'):setValue(768):combo(screenHeights)
graphics('disableAero'):setValue(false):checkbox()
graphics('sync'):setValue(false):checkbox()
graphics('fullScreen'):setValue(false):checkbox()
graphics('TranspSSAA'):setValue(false):checkbox()
graphics('OculusRift'):setValue(true):checkbox()

graphics('clutterMaxDistance'):setValue(0):slider(Range(0, 1500))
graphics('treesVisibility'):setValue(1500):slider(Range(1500, 25000)):low(1500):medium(6000):high(9000)
graphics('preloadRadius'):setValue(150000):slider(Range(100, 150000))
			
viewsCockpit('avionics'):setValue(1):combo({Name('256')					:Value(0),
											Name('512')					:Value(1),
											Name(_('512 every frame'))	:Value(2),
											Name('1024')				:Value(3),
											Name(_('1024 every frame'))	:Value(4),
										})
												
viewsCockpit('mirrors'):setValue(true):checkbox()
viewsCockpit('reflections'):setValue(true):checkbox()
viewsCockpit('russianHud'):setValue(false):checkbox()

-- FIXME:      me_music
-- Mute - ,     
--    ,    ,   (-30)
--     , 
--      (-100 - )
local Range = DbOption.Range

sound('volume'):setValue(0):slider(Range(-30, 0):Mute(-100))
sound('music'):setValue(0):slider(Range(-30, 0):Mute(-100))
sound('gui'):setValue(0):slider(Range(-30, 0):Mute(-100))
sound('world'):setValue(0):slider(Range(-30, 0):Mute(-100))
sound('cockpit'):setValue(0):slider(Range(-30, 0):Mute(-100))
sound('headphones'):setValue(0):slider(Range(-30, 0):Mute(-100))
sound('radioSpeech'):setValue(true):checkbox()
sound('subtitles'):setValue(true):checkbox()
sound('GBreathEffect'):setValue(true):checkbox()

misc('synchronize_controls'):setValue(false):checkbox()
misc('accidental_failures'):setValue(false):checkbox():setEnforceable()
misc('headmove'):setValue(false):checkbox()
misc('f5_nearest_ac'):setValue(false):checkbox()
misc('f10_awacs'):setValue(false):checkbox()
misc('f11_free_camera'):setValue(false):checkbox()
misc('force_feedback_enabled'):setValue(false):checkbox()
misc('Coordinate_Display'):setValue('Lat Long'):combo({	Name(_('Lat Long'))		:Value('Lat Long'),
														Name(_('Lat Long Decimal')):Value('Lat Long Decimal'),
														Name(_('MGRS'))			:Value('MGRS'),
													})
misc('F2_view_effects'):setValue(1):combo({	Name(_('NONE'))        :Value(0),
                                            Name(_('FLOATING'))    :Value(1),
                                            Name(_('SHAKING'))	   :Value(2),
                                        })                                                    
misc('show_pilot_body'):setValue(false):checkbox()

return {
	difficulty		= difficultyOptions,
	graphics		= graphicOptions,
	views			= viewsOptions,
	sound			= soundOptions,
	miscellaneous	= miscOptions,
}
